# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from __future__ import unicode_literals

# Import this before Qt to set the correct API
import ninja_ide  # lint:ok

from PyQt4.QtGui import QApplication
from PyQt4.QtGui import QKeySequence
from PyQt4.QtGui import QTreeWidgetItem
from PyQt4.QtCore import Qt

from ninja_ide import resources
from ninja_ide.gui.dialogs import preferences
from ninja_ide.gui.misc import shortcut_manager
from ninja_tests import gui
from ninja_tests import BaseTest


class PreferencesEditorConfigurationTestCase(BaseTest):

    @classmethod
    def setUpClass(cls):
        cls._app = QApplication([])

    @classmethod
    def tearDownClass(cls):
        del cls._app

    def setUp(self):
        super(PreferencesEditorConfigurationTestCase, self).setUp()
        self.settings = gui.FakeQSettings()
        self.patch(preferences, 'QSettings', lambda: self.settings)
        self.editor_completion = preferences.EditorConfiguration()

    def _check_values(self, errors, indent, checkStyle, marginLine,
                      checkForDocstrings, showMarginLine,
                      errorsInLine, centerOnScroll, checkStyleInline,
                      useTabs, highlightWholeLine, removeTrailingSpaces,
                      showTabsAndSpaces, allowWordWrap):
        self.assertEqual(self.settings.values['errors'], errors)
        self.assertEqual(self.settings.values['indent'], indent)
        self.assertEqual(self.settings.values['checkStyle'], checkStyle)
        self.assertEqual(self.settings.values['marginLine'], marginLine)
        self.assertEqual(self.settings.values['checkForDocstrings'],
                         checkForDocstrings)
        self.assertEqual(self.settings.values['showMarginLine'],
                         showMarginLine)
        self.assertEqual(self.settings.values['errorsInLine'], errorsInLine)
        self.assertEqual(self.settings.values['centerOnScroll'],
                         centerOnScroll)
        self.assertEqual(self.settings.values['checkStyleInline'],
                         checkStyleInline)
        self.assertEqual(self.settings.values['useTabs'], useTabs)
        self.assertEqual(self.settings.values['highlightWholeLine'],
                         highlightWholeLine)
        self.assertEqual(self.settings.values['removeTrailingSpaces'],
                         removeTrailingSpaces)
        self.assertEqual(self.settings.values['showTabsAndSpaces'],
                         showTabsAndSpaces)
        self.assertEqual(self.settings.values['allowWordWrap'], allowWordWrap)

    def test_save(self):
        main = preferences.main_container.MainContainer()
        data = []

        def called():
            data.append(True)

        self.patch(main, 'update_editor_margin_line', called)
        self.patch(preferences.pep8mod, 'refresh_checks', called)
        actions = gui.FakeActions()
        self.patch(preferences.actions, 'Actions', lambda: actions)
        self.editor_completion.save()

        self.assertTrue(actions.reset_editor_flags_executed)
        self.assertEqual('set_tab_usage', actions.func_name)
        self.assertEqual([], preferences.pep8mod.options.ignore)
        self.assertEqual([True, True], data)
        self._check_values(True, 4, True, 80, False, True, True, True, True,
                           False, True, True, True, False)


class PreferencesShortcutManagerConfigurationTestCase(BaseTest):

    @classmethod
    def setUpClass(cls):
        cls._app = QApplication([])

    @classmethod
    def tearDownClass(cls):
        del cls._app

    def setUp(self):
        super(PreferencesShortcutManagerConfigurationTestCase, self).setUp()
        self.settings = gui.FakeQSettings()
        self.patch(shortcut_manager, 'QSettings', lambda: self.settings)
        self.patch(resources, 'QSettings', lambda: self.settings)
        # Test data
        custom_default_data = {
            "New-file": QKeySequence(Qt.CTRL + Qt.Key_N)
        }
        # patch shortcuts with test data
        self.patch(resources, 'CUSTOM_SHORTCUTS', custom_default_data)
        self.shortcuts_manager = shortcut_manager.ShortcutConfiguration()

    def test_load_default_shortcuts(self):
        shorts_count = self.shortcuts_manager.result_widget.topLevelItemCount()
        item = self.shortcuts_manager.result_widget.topLevelItem(0)
        shortcut_keys = item.text(1)
        # Expected data
        expected_key = QKeySequence(Qt.CTRL + Qt.Key_N)
        expected_key_str = expected_key.toString(QKeySequence.NativeText)
        # Just one shortcut should be loaded
        self.assertEqual(shorts_count, 1)
        # The key should be the same as the expected
        self.assertEqual(shortcut_keys, expected_key_str)

    def test_save_shortcuts(self):
        data = []

        def called():
            data.append(True)

        actions = gui.FakeActions()
        setattr(actions, 'update_shortcuts', called)
        self.patch(shortcut_manager.actions, 'Actions', lambda: actions)
        self.shortcuts_manager.result_widget.clear()
        key = QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_N)
        key_str = key.toString(QKeySequence.NativeText)
        tree_data = ["New File", key_str, "New-File"]
        item = QTreeWidgetItem(self.shortcuts_manager.result_widget, tree_data)
        item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
        # Before save there is nothing in QSettings
        self.assertEqual(self.settings.value("New-File", None), None)
        # Save
        self.shortcuts_manager.save()
        # After save there is a value for New-File QSettings
        self.assertEqual(self.settings.values["New-File"], key_str)
        # After save check if NINJA call the update_shortcuts in actios.Actions
        self.assertEqual(data, [True])



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import
from __future__ import unicode_literals
from ninja_ide.gui.editor import highlighter
from PyQt4.QtGui import QFont
import unittest


class HighlighterFormatTestCase(unittest.TestCase):

    def setUp(self):
        self._old_font = "Monospace"
        highlighter.settings.FONT_FAMILY = self._old_font
        self._color = "#000000"
        self._a_format = highlighter.format(self._color)
        b_i_string = "bold italic"
        self._bold_italic = highlighter.format(self._color, b_i_string)
        self._new_font = "Arial"
        highlighter.settings.FONT_FAMILY = self._new_font
        self._new_font_format = highlighter.format(self._color)

    def tearDown(self):
        reload(highlighter)

    def test_format_sets_color(self):
        color_name = self._a_format.foreground().color().name()
        self.assertEqual(self._color, color_name)

    def test_format_sets_bold(self):
        bold = QFont.Bold
        f_weight = self._bold_italic.fontWeight()
        self.assertEqual(bold, f_weight)
        f_weight = self._a_format.fontWeight()
        self.assertNotEqual(bold, f_weight)

    def test_format_sets_italic(self):
        italic = self._bold_italic.fontItalic()
        self.assertTrue(italic)
        italic = self._a_format.fontItalic()
        self.assertFalse(italic)

    def test_format_set_font(self):
        font = self._a_format.fontFamily()
        self.assertEqual(self._old_font, font)
        font = self._new_font_format.fontFamily()
        self.assertEqual(self._new_font, font)


class HighlighterRestyleTestCase(unittest.TestCase):

    def setUp(self):
        highlighter.SDEFAULTS = (("style_key", "scheme_key", "default"),)
        a_scheme = {"scheme_key": "#BBBBBB"}
        highlighter.resources.COLOR_SCHEME = a_scheme
        highlighter.restyle(a_scheme)

    def tearDown(self):
        reload(highlighter)

    def test_scheme_populates_styles(self):
        scolor = highlighter.STYLES["style_key"].foreground().color().name()
        self.assertEqual(scolor.lower(), "#bbbbbb")


class SyntaxUserDataTestCase(unittest.TestCase):

    def setUp(self):
        self.err_sud = highlighter.SyntaxUserData(error=True)
        self.noerr_sud = highlighter.SyntaxUserData(error=False)

    def test_initializes_correctly(self):
        self.assertTrue(self.err_sud.error)
        self.assertFalse(self.noerr_sud.error)
        self.assertEqual(self.err_sud.str_groups, [])
        self.assertEqual(self.noerr_sud.str_groups, [])
        self.assertEqual(self.err_sud.comment_start, -1)
        self.assertEqual(self.noerr_sud.comment_start, -1)

    def test_clears_correctly(self):
        self.err_sud.error = True
        self.err_sud.str_groups = [1, 2, 3]
        self.err_sud.comment_start = 2
        self.err_sud.clear_data()
        self.assertFalse(self.err_sud.error)
        self.assertEqual(self.err_sud.str_groups, [])
        self.assertEqual(self.err_sud.comment_start, -1)

    def test_adds_string(self):
        self.assertNotIn((2, 2), self.err_sud.str_groups)
        self.err_sud.add_str_group(1, 2)
        self.assertIn((2, 2), self.err_sud.str_groups)

    def test_comment_start(self):
        self.assertEqual(self.err_sud.comment_start, -1)
        self.err_sud.comment_start_at(2)
        self.assertEqual(self.err_sud.comment_start, 2)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from __future__ import unicode_literals

import os

from ninja_ide.tools.logger import NinjaLogger
logger = NinjaLogger('ninja_ide.gui.explorer.tree_projects_widget')
DEBUG = logger.debug

from PyQt4.QtGui import QTreeWidget
from PyQt4.QtGui import QTreeWidgetItem
from PyQt4.QtGui import QAbstractItemView
from PyQt4.QtGui import QHeaderView
from PyQt4.QtGui import QInputDialog
from PyQt4.QtGui import QMessageBox
from PyQt4.QtGui import QColor
from PyQt4.QtGui import QBrush
from PyQt4.QtGui import QMenu
from PyQt4.QtGui import QIcon
from PyQt4.QtGui import QStyle
from PyQt4.QtGui import QCursor
from PyQt4.QtCore import Qt
from PyQt4.QtCore import QTimer
from PyQt4.QtCore import SIGNAL
from PyQt4.QtCore import QUrl
from PyQt4.QtGui import QDesktopServices

from ninja_ide import resources
from ninja_ide.core import settings
from ninja_ide.core import file_manager
from ninja_ide.core.filesystem_notifications import NinjaFileSystemWatcher
from ninja_ide.core.filesystem_notifications.base_watcher import ADDED,\
    DELETED, REMOVE, RENAME
from ninja_ide.tools import json_manager
from ninja_ide.tools import ui_tools
from ninja_ide.gui.main_panel import main_container
from ninja_ide.gui.dialogs import project_properties_widget
from ninja_ide.tools.completion import completion_daemon


class TreeProjectsWidget(QTreeWidget):

###############################################################################
# TreeProjectsWidget SIGNALS
###############################################################################

    """
    runProject()
    closeProject(QString)
    closeFilesFromProjectClosed(QString)
    addProjectToConsole(QString)
    removeProjectFromConsole(QString)
    projectPropertiesUpdated(QTreeWidgetItem)
    """

###############################################################################

    #Extra context menu 'all' indicate a menu for ALL LANGUAGES!
    extra_menus = {'all': []}
    #Extra context menu by scope all is for ALL the TREE ITEMS!
    extra_menus_by_scope = {'project': [], 'folder': [], 'file': []}

    images = {
        'py': resources.IMAGES['tree-python'],
        'jpg': resources.IMAGES['tree-image'],
        'png': resources.IMAGES['tree-image'],
        'html': resources.IMAGES['tree-html'],
        'css': resources.IMAGES['tree-css'],
        'ui': resources.IMAGES['designer']}

    def __init__(self, state_index=list()):
        QTreeWidget.__init__(self)

        self.header().setHidden(True)
        self.setSelectionMode(QTreeWidget.SingleSelection)
        self.setAnimated(True)

        self._actualProject = None
        #self._projects -> key: [Item, folderStructure]
        self._projects = {}
        self._loading_items = {}
        self._thread_execution = {}
        self.__enableCloseNotification = True
        self._fileWatcher = NinjaFileSystemWatcher
        self._refresh_projects_queue = []
        self._timer_running = False

        self.header().setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.header().setResizeMode(0, QHeaderView.ResizeToContents)
        self.header().setStretchLastSection(False)

        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.connect(self, SIGNAL(
            "customContextMenuRequested(const QPoint &)"),
            self._menu_context_tree)

        signal_name = "itemClicked(QTreeWidgetItem *, int)"
        # For windows double click instead of single click
        if settings.IS_WINDOWS:
            signal_name = "itemDoubleClicked(QTreeWidgetItem *, int)"

        self.connect(self, SIGNAL(signal_name), self._open_file)
        self.connect(self._fileWatcher, SIGNAL("fileChanged(int, QString)"),
                     self._refresh_project_by_path)
        self.itemExpanded.connect(self._item_expanded)
        self.itemCollapsed.connect(self._item_collapsed)
        self.mute_signals = False
        self.state_index = list()
        self._folding_menu = FoldingContextMenu(self)

    def _item_collapsed(self, tree_item):
        """Store status of item when collapsed"""
        if not self.mute_signals:
            path = tree_item.get_full_path()
            if path in self.state_index:
                path_index = self.state_index.index(path)
                self.state_index.pop(path_index)

    def _item_expanded(self, tree_item):
        """Store status of item when expanded"""
        if not self.mute_signals:
            path = tree_item.get_full_path()
            if path not in self.state_index:
                self.state_index.append(path)

    def shutdown(self):
        self._fileWatcher.shutdown_notification()

    def add_extra_menu(self, menu, lang='all'):
        '''
        Add an extra menu for the given language
        @lang: string with the form 'py', 'php', 'json', etc
        '''
        #remove blanks and replace dots Example(.py => py)
        lang = lang.strip().replace('.', '')
        self.extra_menus.setdefault(lang, [])
        self.extra_menus[lang].append(menu)

    def add_extra_menu_by_scope(self, menu, scope):
        '''
        Add an extra menu for the given language
        @scope: string with the menu scope (all, project, folder, item)
        '''
        if scope.project:
            self.extra_menus_by_scope['project'].append(menu)
        if scope.folder:
            self.extra_menus_by_scope['folder'].append(menu)
        if scope.file:
            self.extra_menus_by_scope['file'].append(menu)

    def _menu_context_tree(self, point):
        index = self.indexAt(point)
        if not index.isValid():
            return

        item = self.itemAt(point)
        handler = None
        menu = QMenu(self)
        if item.isFolder or item.parent() is None:
            self._add_context_menu_for_folders(menu, item)
        elif not item.isFolder:
            self._add_context_menu_for_files(menu, item)
        if item.parent() is None:
            #get the extra context menu for this projectType
            if isinstance(item, ProjectItem):
                return
            handler = settings.get_project_type_handler(item.projectType)
            self._add_context_menu_for_root(menu, item)

        menu.addMenu(self._folding_menu)

        #menu for all items (legacy API)!
        extra_menus = self.extra_menus.get('all', ())
        #menu for all items!
        for m in extra_menus:
            if isinstance(m, QMenu):
                menu.addSeparator()
                menu.addMenu(m)
        #menu for the Project Type(if present)
        if handler:
            for m in handler.get_context_menus():
                if isinstance(m, QMenu):
                    menu.addSeparator()
                    menu.addMenu(m)
        #show the menu!
        menu.exec_(QCursor.pos())

    def _add_context_menu_for_root(self, menu, item):
        menu.addSeparator()
        actionRunProject = menu.addAction(QIcon(
            resources.IMAGES['play']), self.tr("Run Project"))
        self.connect(actionRunProject, SIGNAL("triggered()"),
                     SIGNAL("runProject()"))
        actionMainProject = menu.addAction(self.tr("Set as Main Project"))
        self.connect(actionMainProject, SIGNAL("triggered()"),
                     lambda: self.set_default_project(item))
        if item.addedToConsole:
            actionRemoveFromConsole = menu.addAction(
                self.tr("Remove this Project from the Python Console"))
            self.connect(actionRemoveFromConsole, SIGNAL("triggered()"),
                         self._remove_project_from_console)
        else:
            actionAdd2Console = menu.addAction(
                self.tr("Add this Project to the Python Console"))
            self.connect(actionAdd2Console, SIGNAL("triggered()"),
                         self._add_project_to_console)
        actionProperties = menu.addAction(QIcon(resources.IMAGES['pref']),
                                          self.tr("Project Properties"))
        self.connect(actionProperties, SIGNAL("triggered()"),
                     self.open_project_properties)

        menu.addSeparator()
        action_refresh = menu.addAction(
            self.style().standardIcon(QStyle.SP_BrowserReload),
            self.tr("Refresh Project"))
        self.connect(action_refresh, SIGNAL("triggered()"),
                     self._refresh_project)
        action_close = menu.addAction(
            self.style().standardIcon(QStyle.SP_DialogCloseButton),
            self.tr("Close Project"))
        self.connect(action_close, SIGNAL("triggered()"),
                     self._close_project)
        #menu for the project
        for m in self.extra_menus_by_scope['project']:
            if isinstance(m, QMenu):
                menu.addSeparator()
                menu.addMenu(m)

    def _add_context_menu_for_folders(self, menu, item):
        action_add_file = menu.addAction(QIcon(resources.IMAGES['new']),
                                         self.tr("Add New File"))
        self.connect(action_add_file, SIGNAL("triggered()"),
                     self._add_new_file)
        action_add_folder = menu.addAction(QIcon(
            resources.IMAGES['openProj']), self.tr("Add New Folder"))
        self.connect(action_add_folder, SIGNAL("triggered()"),
                     self._add_new_folder)
        action_create_init = menu.addAction(
            self.tr("Create '__init__' Complete"))
        self.connect(action_create_init, SIGNAL("triggered()"),
                     self._create_init)
        if item.isFolder and (item.parent() is not None):
            action_remove_folder = menu.addAction(self.tr("Remove Folder"))
            self.connect(action_remove_folder, SIGNAL("triggered()"),
                         self._delete_folder)
        #Folders but not the root
        if item.isFolder and item.parent() is not None:
            for m in self.extra_menus_by_scope['folder']:
                if isinstance(m, QMenu):
                    menu.addSeparator()
                    menu.addMenu(m)

    def _add_context_menu_for_files(self, menu, item):
        action_rename_file = menu.addAction(self.tr("Rename File"))
        action_move_file = menu.addAction(self.tr("Move File"))
        action_copy_file = menu.addAction(self.tr("Copy File"))
        action_remove_file = menu.addAction(
            self.style().standardIcon(QStyle.SP_DialogCloseButton),
            self.tr("Delete File"))
        self.connect(action_remove_file, SIGNAL("triggered()"),
                     self._delete_file)
        self.connect(action_rename_file, SIGNAL("triggered()"),
                     self._rename_file)
        self.connect(action_copy_file, SIGNAL("triggered()"),
                     self._copy_file)
        self.connect(action_move_file, SIGNAL("triggered()"),
                     self._move_file)
        #Allow to edit Qt UI files with the appropiate program
        if item.lang() == 'ui':
            action_edit_ui_file = menu.addAction(self.tr("Edit UI File"))
            self.connect(action_edit_ui_file, SIGNAL("triggered()"),
                         self._edit_ui_file)
        #menu per file language (legacy plugin API)!
        for m in self.extra_menus.get(item.lang(), ()):
            if isinstance(m, QMenu):
                menu.addSeparator()
                menu.addMenu(m)
        #menu for files
        for m in self.extra_menus_by_scope['file']:
            if isinstance(m, QMenu):
                menu.addSeparator()
                menu.addMenu(m)

    def _add_project_to_console(self):
        item = self.currentItem()
        if isinstance(item, ProjectTree):
            self.emit(SIGNAL("addProjectToConsole(QString)"), item.path)
            item.addedToConsole = True

    def _remove_project_from_console(self):
        item = self.currentItem()
        if isinstance(item, ProjectTree):
            self.emit(SIGNAL("removeProjectFromConsole(QString)"), item.path)
            item.addedToConsole = False

    def _open_file(self, item, column):
        if item.childCount() == 0 and not item.isFolder:
            fileName = os.path.join(item.path, item.text(column))
            main_container.MainContainer().open_file(fileName)

    def _get_project_root(self, item=None):
        if item is None:
            item = self.currentItem()
        while item is not None and item.parent() is not None:
            item = item.parent()
        return item

    def set_default_project(self, item):
        item.setForeground(0, QBrush(QColor(0, 204, 82)))
        if self._actualProject:
            item.setForeground(0, QBrush(QColor(255, 165, 0)))
        self._actualProject = item

    def open_project_properties(self):
        item = self._get_project_root()
        proj = project_properties_widget.ProjectProperties(item, self)
        proj.show()

    def _timeout(self):
        projects = set(self._refresh_projects_queue)
        self._refresh_projects_queue = []
        self._timer_running = False
        for prefresh in projects:
            self._refresh_project(prefresh)

    def _refresh_project_by_path(self, event, folder):
        if event not in (DELETED, ADDED, REMOVE, RENAME):
            return
        oprojects = self.get_open_projects()
        for each_project in oprojects:
            p_path = each_project.path
            if file_manager.belongs_to_folder(p_path, folder) and \
               file_manager.is_supported_extension(folder,
                                                   each_project.extensions) \
               and folder[:1] != '.':
                self._refresh_projects_queue.append(each_project)
                break
        if not self._timer_running:
            self._timeout()
            QTimer.singleShot(3000, self._timeout)
            self._timer_running = True

    def _refresh_project(self, item=None):
        if item is None:
            item = self.currentItem()
        parentItem = self._get_project_root(item)
        if parentItem is None:
            return
        if item.parent() is None:
            path = item.path
        else:
            path = file_manager.create_path(item.path, item.text(0))

        thread = ui_tools.ThreadProjectExplore()
        self._thread_execution[path] = thread
        self.connect(thread, SIGNAL("folderDataRefreshed(PyQt_PyObject)"),
                     self._callback_refresh_project)
        self.connect(thread, SIGNAL("finished()"), self._clean_threads)
        thread.refresh_project(path, item, parentItem.extensions)

    def _clean_threads(self):
        paths_to_delete = []
        for path in self._thread_execution:
            thread = self._thread_execution.get(path, None)
            if thread and not thread.isRunning():
                paths_to_delete.append(path)
        for path in paths_to_delete:
            thread = self._thread_execution.pop(path, None)
            if thread:
                thread.wait()

    def _callback_refresh_project(self, value):
        path, item, structure = value
        item.takeChildren()
        self._load_folder(structure, path, item)
        #todo: refresh completion
        item.setExpanded(True)
        if isinstance(item, ProjectTree):
            self.emit(SIGNAL("projectPropertiesUpdated(QTreeWidgetItem)"),
                      item)

    def _close_project(self):
        item = self.currentItem()
        index = self.indexOfTopLevelItem(item)
        pathKey = item.path
        self._fileWatcher.remove_watch(pathKey)
        self.takeTopLevelItem(index)
        self._projects.pop(pathKey, None)
        if self.__enableCloseNotification:
            self.emit(SIGNAL("closeProject(QString)"), pathKey)
        self.emit(SIGNAL("closeFilesFromProjectClosed(QString)"), pathKey)
        item = self.currentItem()
        if item:
            self.set_default_project(item)
            self._actualProject = item
        else:
            self._actualProject = None

    def _create_init(self):
        item = self.currentItem()
        if item.parent() is None:
            pathFolder = item.path
        else:
            pathFolder = os.path.join(item.path, str(item.text(0)))
        try:
            file_manager.create_init_file_complete(pathFolder)
        except file_manager.NinjaFileExistsException as ex:
            QMessageBox.information(self, self.tr("Create INIT fail"),
                                    ex.message)
        self._refresh_project(item)

    def _add_new_file(self):
        item = self.currentItem()
        if item.parent() is None:
            pathForFile = item.path
        else:
            pathForFile = os.path.join(item.path, item.text(0))
        result = QInputDialog.getText(self, self.tr("New File"),
                                      self.tr("Enter the File Name:"))
        fileName = result[0]

        if result[1] and fileName.strip() != '':
            try:
                fileName = os.path.join(pathForFile, fileName)
                fileName = file_manager.store_file_content(
                    fileName, '', newFile=True)
                name = file_manager.get_basename(fileName)
                subitem = ProjectItem(item, name, pathForFile)
                subitem.setToolTip(0, name)
                subitem.setIcon(0, self._get_file_icon(name))
                mainContainer = main_container.MainContainer()
                mainContainer.open_file(fileName)
            except file_manager.NinjaFileExistsException as ex:
                QMessageBox.information(self, self.tr("File Already Exists"),
                    (self.tr("Invalid Path: the file '%s' already exists.") %
                     ex.filename))

    def add_existing_file(self, path):
        relative = file_manager.convert_to_relative(
            self._actualProject.path, path)
        paths = relative.split(os.sep)[:-1]
        itemParent = self._actualProject
        for p in paths:
            for i in range(itemParent.childCount()):
                item = itemParent.child(i)
                if item.text(0) == p:
                    itemParent = item
                    break
        itemParent.setSelected(True)
        name = file_manager.get_basename(path)
        subitem = ProjectItem(itemParent, name, file_manager.get_folder(path))
        subitem.setToolTip(0, name)
        subitem.setIcon(0, self._get_file_icon(name))
        itemParent.setExpanded(True)

    def _add_new_folder(self):
        item = self.currentItem()
        if item.parent() is None:
            pathForFolder = item.path
        else:
            pathForFolder = os.path.join(item.path, item.text(0))
        result = QInputDialog.getText(self, self.tr("New Folder"),
                                      self.tr("Enter the Folder Name:"))
        folderName = result[0]

        if result[1] and folderName.strip() != '':
            folderName = os.path.join(pathForFolder, folderName)
            file_manager.create_folder(folderName)
            name = file_manager.get_basename(folderName)
            subitem = ProjectItem(item, name, pathForFolder)
            subitem.setToolTip(0, name)
            subitem.setIcon(0, QIcon(resources.IMAGES['tree-folder']))
            self._refresh_project(item)

    def _delete_file(self):
        item = self.currentItem()
        val = QMessageBox.question(self, self.tr("Delete File"),
                       self.tr("Do you want to delete the following file: ")
                       + os.path.join(item.path, item.text(0)),
                       QMessageBox.Yes, QMessageBox.No)
        if val == QMessageBox.Yes:
            path = file_manager.create_path(item.path, item.text(0))
            file_manager.delete_file(item.path, item.text(0))
            index = item.parent().indexOfChild(item)
            item.parent().takeChild(index)
            mainContainer = main_container.MainContainer()
            if mainContainer.is_open(path):
                mainContainer.close_deleted_file(path)

    def _delete_folder(self):
        item = self.currentItem()
        val = QMessageBox.question(self, self.tr("Delete Folder"),
                       self.tr("Do you want to delete the following folder: ")
                       + os.path.join(item.path, item.text(0)),
                       QMessageBox.Yes, QMessageBox.No)
        if val == QMessageBox.Yes:
            file_manager.delete_folder(item.path, item.text(0))
            index = item.parent().indexOfChild(item)
            item.parent().takeChild(index)

    def _rename_file(self):
        item = self.currentItem()
        if item.parent() is None:
            pathForFile = item.path
        else:
            pathForFile = os.path.join(item.path, item.text(0))
        result = QInputDialog.getText(self, self.tr("Rename File"),
                          self.tr("Enter New File Name:"), text=item.text(0))
        fileName = result[0]

        if result[1] and fileName.strip() != '':
            fileName = os.path.join(
                file_manager.get_folder(pathForFile), fileName)
            if pathForFile == fileName:
                return
            try:
                fileName = file_manager.rename_file(pathForFile, fileName)
                name = file_manager.get_basename(fileName)
                mainContainer = main_container.MainContainer()
                if mainContainer.is_open(pathForFile):
                    mainContainer.change_open_tab_name(pathForFile, fileName)
                subitem = ProjectItem(item.parent(), name,
                                      file_manager.get_folder(fileName))
                subitem.setToolTip(0, name)
                subitem.setIcon(0, self._get_file_icon(name))
                index = item.parent().indexOfChild(item)
                subitem.parent().takeChild(index)
            except file_manager.NinjaFileExistsException as ex:
                QMessageBox.information(self, self.tr("File Already Exists"),
                    (self.tr("Invalid Path: the file '%s' already exists.") %
                     ex.filename))

    def _copy_file(self):
        #get the selected QTreeWidgetItem
        item = self.currentItem()
        if item.parent() is None:
            pathForFile = item.path
        else:
            pathForFile = os.path.join(item.path, item.text(0))
        pathProjects = [p.path for p in self.get_open_projects()]
        addToProject = ui_tools.AddToProject(pathProjects, self)
        addToProject.setWindowTitle(self.tr("Copy File to"))
        addToProject.exec_()
        if not addToProject.pathSelected:
            return
        name = QInputDialog.getText(self, self.tr("Copy File"),
                                    self.tr("File Name:"), text=item.text(0))[0]
        if not name:
            QMessageBox.information(self, self.tr("Invalid Name"),
                        self.tr("The file name is empty, please enter a name"))
            return
        path = file_manager.create_path(addToProject.pathSelected, name)
        try:
            content = file_manager.read_file_content(pathForFile)
            path = file_manager.store_file_content(path, content, newFile=True)
            self.add_existing_file(path)
        except file_manager.NinjaFileExistsException as ex:
                QMessageBox.information(self, self.tr("File Already Exists"),
                    (self.tr("Invalid Path: the file '%s' already exists.") %
                     ex.filename))

    def _move_file(self):
        item = self.currentItem()
        if item.parent() is None:
            pathForFile = item.path
        else:
            pathForFile = os.path.join(item.path, item.text(0))
        pathProjects = [p.path for p in self.get_open_projects()]
        addToProject = ui_tools.AddToProject(pathProjects, self)
        addToProject.setWindowTitle(self.tr("Copy File to"))
        addToProject.exec_()
        if not addToProject.pathSelected:
            return
        name = file_manager.get_basename(pathForFile)
        path = file_manager.create_path(addToProject.pathSelected, name)
        try:
            content = file_manager.read_file_content(pathForFile)
            path = file_manager.store_file_content(path, content, newFile=True)
            file_manager.delete_file(pathForFile)
            index = item.parent().indexOfChild(item)
            item.parent().takeChild(index)
            self.add_existing_file(path)
            # Update path of opened file
            main = main_container.MainContainer()
            if main.is_open(pathForFile):
                widget = main.get_widget_for_path(pathForFile)
                if widget:
                    widget.ID = path
        except file_manager.NinjaFileExistsException as ex:
                QMessageBox.information(self, self.tr("File Already Exists"),
                    (self.tr("Invalid Path: the file '%s' already exists.") %
                     ex.filename))

    def _edit_ui_file(self):
        item = self.currentItem()
        if item.parent() is None:
            pathForFile = item.path
        else:
            pathForFile = os.path.join(item.path, item.text(0))
        pathForFile = "file://%s" % pathForFile
        #open the correct program to edit Qt UI files!
        QDesktopServices.openUrl(QUrl(pathForFile, QUrl.TolerantMode))

    def loading_project(self, folder, reload_item=None):
        loadingItem = ui_tools.LoadingItem()
        if reload_item is None:
            parent = self
        else:
            parent = reload_item.parent()
        item = loadingItem.add_item_to_tree(folder, self, ProjectItem, parent)
        self._loading_items[folder] = item

    def remove_loading_icon(self, folder):
        item = self._loading_items.pop(folder, None)
        if item is not None:
            index = self.indexOfTopLevelItem(item)
            self.takeTopLevelItem(index)

    def load_project(self, folderStructure, folder):
        if not folder:
            return

        self.remove_loading_icon(folder)

        name = file_manager.get_basename(folder)
        item = ProjectTree(self, name, folder)
        item.isFolder = True
        item.setToolTip(0, folder)
        item.setIcon(0, QIcon(resources.IMAGES['tree-app']))
        self._projects[folder] = item
        if folderStructure[folder][1] is not None:
            folderStructure[folder][1].sort()
        self._load_folder(folderStructure, folder, item)
        item.setExpanded(True)
        if len(self._projects) == 1:
            self.set_default_project(item)
        if self.currentItem() is None:
            item.setSelected(True)
            self.setCurrentItem(item)
        self._fileWatcher.add_watch(folder)
        completion_daemon.add_project_folder(folder)
        self.sortItems(0, Qt.AscendingOrder)

    def _load_folder(self, folderStructure, folder, parentItem):
        """Load the Tree Project structure recursively."""
        # Avoid failing if for some reason folder is not found
        # Might be the case of special files, as links or versioning files
        files, folders = folderStructure.get(folder, [None, None])

        if files is not None:
            files.sort()
            for i in files:
                subitem = ProjectItem(parentItem, i, folder)
                subitem.setToolTip(0, i)
                subitem.setIcon(0, self._get_file_icon(i))
        if folders is not None:
            folders.sort()
            for _folder in folders:
                if _folder.startswith('.'):
                    continue
                subfolder = ProjectItem(parentItem, _folder, folder, True)
                subfolder.setToolTip(0, _folder)
                subfolder.setIcon(0, QIcon(resources.IMAGES['tree-folder']))
                subFolderPath = os.path.join(folder, _folder)
                if subFolderPath in self.state_index:
                    subfolder.setExpanded(True)
                self._load_folder(folderStructure,
                                  subFolderPath, subfolder)

    def _get_file_icon(self, fileName):
        return QIcon(self.images.get(file_manager.get_file_extension(fileName),
                                     resources.IMAGES['tree-generic']))

    def get_item_for_path(self, path):
        items = self.findItems(file_manager.get_basename(path),
                               Qt.MatchRecursive, 0)
        folder = file_manager.get_folder(path)
        for item in items:
            if file_manager.belongs_to_folder(folder, item.path):
                return item

    def get_project_by_name(self, projectName):
        """Return the name of the project item based on the project name."""
        # Return the item or None if it's not found
        for item in list(self._projects.values()):
            if item.name == projectName:
                return item

    def get_selected_project_path(self):
        if self._actualProject:
            return self._actualProject.path
        return None

    def get_project_main_file(self):
        if self._actualProject:
            return self._actualProject.mainFile
        return ''

    def get_selected_project_type(self):
        rootItem = self._get_project_root()
        return rootItem.projectType

    def get_selected_project_lang(self):
        rootItem = self._get_project_root()
        return rootItem.lang()

    def get_open_projects(self):
        return list(self._projects.values())

    def is_open(self, path):
        return len([True for item in list(self._projects.values())
                    if item.path == path]) != 0

    def _set_current_project(self, path):
        for item in list(self._projects.values()):
            if item.path == path:
                self.set_default_project(item)
                break

    def _close_open_projects(self):
        self.__enableCloseNotification = False
        for i in range(self.topLevelItemCount()):
            self.setCurrentItem(self.topLevelItem(0))
            self._close_project()
        self.__enableCloseNotification = True
        self._projects = {}

    def keyPressEvent(self, event):
        item = self.currentItem()
        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
            self._open_file(item, 0)
        elif event.key() in (Qt.Key_Space, Qt.Key_Slash) and item.isFolder:
            expand = not item.isExpanded()
            item.setExpanded(expand)
        elif event.key() == Qt.Key_Left and not item.isExpanded():
            parent = item.parent()
            if parent:
                parent.setExpanded(False)
                self.setCurrentItem(parent)
                return
        super(TreeProjectsWidget, self).keyPressEvent(event)


class ProjectItem(QTreeWidgetItem):

    def __init__(self, parent, name, path, isFolder=False):
        QTreeWidgetItem.__init__(self, parent)
        self.setText(0, name)
        self.path = path
        self.isFolder = isFolder

    @property
    def isProject(self):
        #flag to check if the item is a project ALWAYS FALSE
        return False

    def lang(self):
        return file_manager.get_file_extension(self.text(0))

    def get_full_path(self):
        '''
        Returns the full path of the file
        '''
        return os.path.join(self.path, self.text(0))

    def set_item_icon(self, icon):
        self.setIcon(0, icon)

    def __lt__(self, otherItem):
        column = self.treeWidget().sortColumn()
        my_text = ('1%s' % self.text(column).lower() if
                   self.isFolder else '0%s' % self.text(column).lower())
        other_text = ('1%s' % otherItem.text(column).lower() if
                      otherItem.isFolder else '0%s'
                      % otherItem.text(column).lower())
        return my_text < other_text


class ProjectTree(QTreeWidgetItem):

    def __init__(self, parent, _name, path):
        QTreeWidgetItem.__init__(self, parent)
        self._parent = parent
        self.setText(0, _name)
        self.path = path
        self.isFolder = True
        self.setForeground(0, QBrush(QColor(255, 165, 0)))
        project = json_manager.read_ninja_project(path)
        self.name = project.get('name', '')
        if self.name == '':
            self.name = _name
        self.setText(0, self.name)
        self.projectType = project.get('project-type', '')
        self.description = project.get('description', '')
        self.url = project.get('url', '')
        self.license = project.get('license', '')
        self.mainFile = project.get('mainFile', '')
        self.preExecScript = project.get('preExecScript', '')
        self.postExecScript = project.get('postExecScript', '')
        self.indentation = project.get('indentation', settings.INDENT)
        self.useTabs = project.get('use-tabs', settings.USE_TABS)
        self.extensions = project.get('supported-extensions',
                                      settings.SUPPORTED_EXTENSIONS)
        self.pythonPath = project.get('pythonPath', settings.PYTHON_PATH)
        self.PYTHONPATH = project.get('PYTHONPATH', '')
        self.additional_builtins = project.get('additional_builtins', [])
        self.programParams = project.get('programParams', '')
        self.venv = project.get('venv', '')
        self.related_projects = project.get('relatedProjects', [])
        self.update_paths()
        self.addedToConsole = False

    def update_paths(self):
        for path in self.related_projects:
            completion_daemon.add_project_folder(path)

    @property
    def isProject(self):
        #flag to check if the item is a project ALWAYS TRUE
        return True

    def lang(self):
        if self.mainFile != '':
            return file_manager.get_file_extension(self.mainFile)
        return 'py'

    def get_full_path(self):
        '''
        Returns the full path of the project
        '''
        project_file = json_manager.get_ninja_project_file(self.path)
        if not project_file:  # FIXME: If we dont have a project file
            project_file = ''     # we should do SOMETHING! like kill zombies!
        return os.path.join(self.path, project_file)


class FoldingContextMenu(QMenu):
    """
    This class represents a menu for Folding/Unfolding task
    """

    def __init__(self, tree):
        super(FoldingContextMenu, self).__init__()
        self.setTitle(self.tr("Fold/Unfold"))
        self._tree = tree
        fold_project = self.addAction(self.tr("Fold the project"))
        unfold_project = self.addAction(self.tr("Unfold the project"))
        self.addSeparator()
        fold_all_projects = self.addAction(self.tr("Fold all projects"))
        unfold_all_projects = self.addAction(self.tr("Unfold all projects"))

        self.connect(fold_project, SIGNAL("triggered()"),
                     lambda: self._fold_unfold_project(False))
        self.connect(unfold_project, SIGNAL("triggered()"),
                     lambda: self._fold_unfold_project(True))
        self.connect(fold_all_projects, SIGNAL("triggered()"),
                     self._fold_all_projects)
        self.connect(unfold_all_projects, SIGNAL("triggered()"),
                     self._unfold_all_projects)

    def _recursive_fold_unfold(self, item, expand):
        if item.isFolder:
            item.setExpanded(expand)
        for index in range(item.childCount()):
            child = item.child(index)
            self._recursive_fold_unfold(child, expand)

    def _fold_unfold_project(self, expand):
        """
        Fold the current project
        """
        root = self._tree._get_project_root(item=self._tree.currentItem())
        childs = root.childCount()
        if childs:
            root.setExpanded(expand)

        for index in range(childs):
            item = root.child(index)
            self._recursive_fold_unfold(item, expand)

    def _fold_all_projects(self):
        """
        Fold all projects
        """
        self._tree.collapseAll()

    def _unfold_all_projects(self):
        """
        Unfold all project
        """
        self._tree.expandAll()



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import

from PyQt4.QtGui import QWidget
from PyQt4.QtGui import QVBoxLayout
from PyQt4.QtWebKit import QWebView
from PyQt4.QtCore import QUrl
from PyQt4.QtWebKit import QWebSettings


class WebRender(QWidget):

    def __init__(self):
        QWidget.__init__(self)

        vbox = QVBoxLayout(self)
        #Web Frame
        self.webFrame = QWebView()
        QWebSettings.globalSettings().setAttribute(
            QWebSettings.DeveloperExtrasEnabled, True)
        vbox.addWidget(self.webFrame)

    def render_page(self, url):
        self.webFrame.load(QUrl('file:///' + url))

    def render_from_html(self, html, url=None):
        url = url and QUrl(url) or ""
        self.webFrame.setHtml(html, url)



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

###############################################################################
# METADATA
###############################################################################

__prj__ = "NINJA-IDE"
__author__ = "The NINJA-IDE Team"
__mail__ = "ninja-ide at googlegroups dot com"
__url__ = "http://www.ninja-ide.org"
__source__ = "https://github.com/ninja-ide/ninja-ide"
__version__ = "2.3"
__license__ = "GPL3"

###############################################################################
# DOC
###############################################################################

"""NINJA-IDE is a cross-platform integrated development environment (IDE).
NINJA-IDE runs on Linux/X11, Mac OS X and Windows desktop operating systems,
and allows developers to create applications for several purposes using all the
tools and utilities of NINJA-IDE, making the task of writing software easier
and more enjoyable.
"""


###############################################################################
# SET PYQT API 2
###############################################################################

import sip
API_NAMES = ["QDate", "QDateTime", "QString", "QTime", "QUrl", "QTextStream",
             "QVariant"]
API_VERSION = 2
for name in API_NAMES:
    sip.setapi(name, API_VERSION)


###############################################################################
# START
###############################################################################


def setup_and_run():
    # import only on run
    # Dont import always this, setup.py will fail
    from ninja_ide import core
    from multiprocessing import freeze_support

    # Used to support multiprocessing on windows packages
    freeze_support()

    # Run NINJA-IDE
    core.run_ninja()



# -*- coding: utf-8 -*-
import os
from urlparse import urlparse, urlunparse

from PyQt4.QtGui import QWidget
from PyQt4.QtGui import QVBoxLayout
from PyQt4.QtCore import QUrl
from PyQt4.QtCore import QDir
from PyQt4.QtCore import QSettings
from PyQt4.QtCore import SIGNAL
from PyQt4.QtDeclarative import QDeclarativeView

from ninja_ide import resources
from ninja_ide.gui.main_panel import itab_item


class StartPage(QWidget, itab_item.ITabItem):

    def __init__(self, parent=None):
        super(StartPage, self).__init__(parent)
        self._id = "Start Page"
        vbox = QVBoxLayout(self)
        self.view = QDeclarativeView()
        self.view.setMinimumWidth(400)
        self.view.setResizeMode(QDeclarativeView.SizeRootObjectToView)
        path_qml = QDir.fromNativeSeparators(
            os.path.join(resources.QML_FILES, "StartPage.qml"))
        path_qml = urlunparse(urlparse(path_qml)._replace(scheme='file'))
        self.view.setSource(QUrl(path_qml))
        self.root = self.view.rootObject()
        vbox.addWidget(self.view)

        self.load_items()

        self.connect(self.root, SIGNAL("openProject(QString)"),
            self._open_project)
        self.connect(self.root, SIGNAL("removeProject(QString)"),
            self._on_click_on_delete)
        self.connect(self.root, SIGNAL("markAsFavorite(QString, bool)"),
            self._on_click_on_favorite)
        self.connect(self.root, SIGNAL("openPreferences()"),
            lambda: self.emit(SIGNAL("openPreferences()")))

    def _open_project(self, path):
        self.emit(SIGNAL("openProject(QString)"), path)

    def _on_click_on_delete(self, path):
        settings = QSettings(resources.SETTINGS_PATH, QSettings.IniFormat)
        recent_projects = settings.value("recentProjects")
        if path in recent_projects:
            del recent_projects[path]
            settings.setValue("recentProjects", recent_projects)

    def _on_click_on_favorite(self, path, value):
        settings = QSettings(resources.SETTINGS_PATH, QSettings.IniFormat)
        recent_projects = settings.value("recentProjects")
        properties = recent_projects[path]
        properties["isFavorite"] = value
        recent_projects[path] = properties
        settings.setValue("recentProjects", recent_projects)

    def load_items(self):
        settings = QSettings(resources.SETTINGS_PATH, QSettings.IniFormat)
        listByFavorites = []
        listNoneFavorites = []
        recent_projects_dict = dict(settings.value('recentProjects', {}))
        #Filter for favorites
        for recent_project_path, content in list(recent_projects_dict.items()):
            if bool(dict(content)["isFavorite"]):
                listByFavorites.append((recent_project_path,
                    content["lastopen"]))
            else:
                listNoneFavorites.append((recent_project_path,
                    content["lastopen"]))
        if len(listByFavorites) > 1:
            # sort by date favorites
            listByFavorites = sorted(listByFavorites,
                key=lambda date: listByFavorites[1])

        if len(listNoneFavorites) > 1:
            #sort by date last used
            listNoneFavorites = sorted(listNoneFavorites,
                key=lambda date: listNoneFavorites[1])

        for recent_project_path in listByFavorites:
            path = recent_project_path[0]
            name = recent_projects_dict[path]['name']
            self.root.add_project(name, path, True)

        for recent_project_path in listNoneFavorites:
            path = recent_project_path[0]
            name = recent_projects_dict[path]['name']
            self.root.add_project(name, path, False)


# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import

import sys
from code import InteractiveConsole


class Cache(object):
    """Replace stdout and stderr behavior in order to collect outputs."""

    def __init__(self):
        self.reset()

    def reset(self):
        """Clean the cache."""
        self.out = []

    def write(self, line):
        """Collect the output into cache to be accesed later."""
        self.out.append(line)

    def flush(self):
        """Join together all the outputs and return it to be displayed."""
        if len(self.out) > 1:
            output = ''.join(self.out)[:-1]
            self.reset()
            return output


class ExitWrapper(object):
    """Wrap the built-in function exit in the interpreter."""

    def __call__(self, code=None):
        return None

    def __repr__(self):
        return exit.__repr__()


class HelpWrapper(object):
    """Wrap the built-in function help in the interpreter."""

    def __call__(self, *args, **kwds):
        if args or kwds:
            return help(*args, **kwds)

    def __repr__(self):
        return help.__repr__()


class Console(InteractiveConsole):
    """Work as a Python Console."""

    def __init__(self):
        InteractiveConsole.__init__(self, locals={'exit': ExitWrapper(),
                                                  'help': HelpWrapper()})
        self.stdout = sys.stdout
        self.stderr = sys.stderr
        self._cache = Cache()
        self.output = ''

    def get_output(self):
        """Replace out and error channels with cache."""
        sys.stdout = self._cache
        sys.stderr = self._cache

    def return_output(self):
        """Reassign the proper values to output and error channel."""
        sys.stdout = self.stdout
        sys.stderr = self.stderr

    def push(self, line):
        """Insert a command into the console."""
        self.get_output()
        val = InteractiveConsole.push(self, line)
        self.return_output()
        self.output = self._cache.flush()
        return val

    def get_type(self, var):
        """Get the type of a variable."""
        type_line = "'.'.join([type(%s).__module__, type(%s).__name__])" % \
            (var[:-1], var[:-1])
        self.get_output()
        InteractiveConsole.push(self, type_line)
        self.return_output()
        exec_line = self._cache.flush() + '.'
        exec_line = exec_line.replace("'", "")
        return exec_line



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import

from PyQt4.QtGui import QIcon
from PyQt4.QtGui import QTextCursor
from PyQt4.QtGui import QKeySequence
from PyQt4.QtCore import SIGNAL
from PyQt4.QtCore import QObject

from ninja_ide import resources
from ninja_ide.core import settings
from ninja_ide.gui.main_panel import main_container
from ninja_ide.gui.misc import misc_container
from ninja_ide.gui import status_bar
from ninja_ide.gui.dialogs import preferences


class MenuEdit(QObject):

    def __init__(self, menuEdit, toolbar):
        QObject.__init__(self)

        undoAction = menuEdit.addAction(QIcon(resources.IMAGES['undo']),
            (self.trUtf8("Undo (%s+Z)") % settings.OS_KEY))
        redoAction = menuEdit.addAction(QIcon(resources.IMAGES['redo']),
            (self.trUtf8("Redo (%s)") % resources.get_shortcut("Redo").toString(
                    QKeySequence.NativeText)))
        cutAction = menuEdit.addAction(QIcon(resources.IMAGES['cut']),
            (self.trUtf8("&Cut (%s+X)") % settings.OS_KEY))
        copyAction = menuEdit.addAction(QIcon(resources.IMAGES['copy']),
            (self.trUtf8("&Copy (%s+C)") % settings.OS_KEY))
        pasteAction = menuEdit.addAction(QIcon(resources.IMAGES['paste']),
            (self.trUtf8("&Paste (%s+V)") % settings.OS_KEY))
        menuEdit.addSeparator()
        findAction = menuEdit.addAction(QIcon(resources.IMAGES['find']),
            (self.trUtf8("Find (%s)") % resources.get_shortcut("Find").toString(
                    QKeySequence.NativeText)))
        findReplaceAction = menuEdit.addAction(
            QIcon(resources.IMAGES['findReplace']),
            (self.trUtf8("Find/Replace (%s)") %
                resources.get_shortcut("Find-replace").toString(
                    QKeySequence.NativeText)))
        findWithWordAction = menuEdit.addAction(
            (self.trUtf8("Find using word under cursor (%s)") %
                resources.get_shortcut("Find-with-word").toString(
                    QKeySequence.NativeText)))
        findInFilesAction = menuEdit.addAction(QIcon(resources.IMAGES['find']),
            (self.trUtf8("Find in Files (%s)") %
                resources.get_shortcut("Find-in-files").toString(
                    QKeySequence.NativeText)))
        locatorAction = menuEdit.addAction(QIcon(resources.IMAGES['locator']),
            (self.trUtf8("Code Locator (%s)") %
                resources.get_shortcut("Code-locator").toString(
                    QKeySequence.NativeText)))
        menuEdit.addSeparator()
        upperAction = menuEdit.addAction(
            self.trUtf8("Convert selected Text to: UPPER"))
        lowerAction = menuEdit.addAction(
            self.trUtf8("Convert selected Text to: lower"))
        titleAction = menuEdit.addAction(
            self.trUtf8("Convert selected Text to: Title Word"))
        menuEdit.addSeparator()
        prefAction = menuEdit.addAction(QIcon(resources.IMAGES['pref']),
            self.trUtf8("Preference&s"))

        self.toolbar_items = {
            'undo': undoAction,
            'redo': redoAction,
            'cut': cutAction,
            'copy': copyAction,
            'paste': pasteAction,
            'find': findAction,
            'find-replace': findReplaceAction,
            'find-files': findInFilesAction,
            'code-locator': locatorAction}

        self.connect(cutAction, SIGNAL("triggered()"), self._editor_cut)
        self.connect(copyAction, SIGNAL("triggered()"), self._editor_copy)
        self.connect(pasteAction, SIGNAL("triggered()"), self._editor_paste)
        self.connect(redoAction, SIGNAL("triggered()"), self._editor_redo)
        self.connect(undoAction, SIGNAL("triggered()"), self._editor_undo)
        self.connect(upperAction, SIGNAL("triggered()"), self._editor_upper)
        self.connect(lowerAction, SIGNAL("triggered()"), self._editor_lower)
        self.connect(titleAction, SIGNAL("triggered()"), self._editor_title)
        self.connect(findAction, SIGNAL("triggered()"),
            status_bar.StatusBar().show)
        self.connect(findWithWordAction, SIGNAL("triggered()"),
            status_bar.StatusBar().show_with_word)
        self.connect(findReplaceAction, SIGNAL("triggered()"),
            status_bar.StatusBar().show_replace)
        self.connect(findInFilesAction, SIGNAL("triggered()"),
            self._show_find_in_files)
        self.connect(locatorAction, SIGNAL("triggered()"),
            status_bar.StatusBar().show_locator)
        self.connect(prefAction, SIGNAL("triggered()"), self._show_preferences)

    def _editor_upper(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.textCursor().beginEditBlock()
            if editorWidget.textCursor().hasSelection():
                text = editorWidget.textCursor().selectedText().upper()
            else:
                text = editorWidget._text_under_cursor().upper()
                editorWidget.moveCursor(QTextCursor.StartOfWord)
                editorWidget.moveCursor(QTextCursor.EndOfWord,
                    QTextCursor.KeepAnchor)
            editorWidget.textCursor().insertText(text)
            editorWidget.textCursor().endEditBlock()

    def _editor_lower(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.textCursor().beginEditBlock()
            if editorWidget.textCursor().hasSelection():
                text = editorWidget.textCursor().selectedText().lower()
            else:
                text = editorWidget._text_under_cursor().lower()
                editorWidget.moveCursor(QTextCursor.StartOfWord)
                editorWidget.moveCursor(QTextCursor.EndOfWord,
                    QTextCursor.KeepAnchor)
            editorWidget.textCursor().insertText(text)
            editorWidget.textCursor().endEditBlock()

    def _editor_title(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.textCursor().beginEditBlock()
            if editorWidget.textCursor().hasSelection():
                text = editorWidget.textCursor().selectedText().title()
            else:
                text = editorWidget._text_under_cursor().title()
                editorWidget.moveCursor(QTextCursor.StartOfWord)
                editorWidget.moveCursor(QTextCursor.EndOfWord,
                    QTextCursor.KeepAnchor)
            editorWidget.textCursor().insertText(text)
            editorWidget.textCursor().endEditBlock()

    def _editor_cut(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.cut()

    def _editor_copy(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.copy()

    def _editor_paste(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.paste()

    def _editor_redo(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.redo()

    def _editor_undo(self):
        editorWidget = main_container.MainContainer().get_actual_editor()
        if editorWidget:
            editorWidget.undo()

    def _show_preferences(self):
        pref = preferences.PreferencesWidget(main_container.MainContainer())
        pref.show()

    def _show_find_in_files(self):
        misc_container.MiscContainer().show_find_in_files_widget()



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import

import _ast
import unittest

from ninja_ide.tools.completion import analyzer
from ninja_ide.tools.completion import model
from ninja_tests.tools.completion import SOURCE_ANALYZER_NATIVE


class AnalyzerTestCase(unittest.TestCase):

    def setUp(self):
        self.analyzer = analyzer.Analyzer()

###############################################################################
# SIMPLE NATIVE TYPES
###############################################################################

    def test_content_value(self):
        self.assertTrue(self.analyzer.content is None)
        self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)
        self.assertTrue(self.analyzer.content is None)

    def test_imports(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        type1 = model._TypeData(None, 'sys', 'import sys', None)
        type2 = model._TypeData(None, 'os', 'import os', None)
        type3 = model._TypeData(None, 'sys.exit', 'import sys.exit', None)
        expected = {'sys': type1, 'os': type2, 'exit': type3}

        for imp in module.imports:
            data = expected[imp]
            impo = module.imports[imp]
            self.assertEqual(data.data_type, impo.data_type)
            self.assertEqual(data.line_content, impo.line_content)

    def test_module_class_func_attrs(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        result_f = module.functions.keys()
        result_f.sort()
        result_a = module.attributes.keys()
        result_a.sort()
        result_c = module.classes.keys()
        functions = ['global_func']
        attrs = ['a', 'b', 'man']
        classes = ['Test']
        functions.sort()
        attrs.sort()
        self.assertEqual(result_f, functions)
        self.assertEqual(result_a, attrs)
        self.assertEqual(result_c, classes)

    def test_attrs_in_module_func(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        func = module.functions['global_func']

        self.assertEqual(func.args, {})
        self.assertEqual(func.decorators, [])
        self.assertEqual(func.return_type, [])
        self.assertEqual(func.name, 'global_func')
        self.assertEqual(func.functions, {})
        #Assign: 1
        result_a = func.attributes.keys()
        result_a.sort()
        expected = ['another', 'bo', 'di', 'obj']
        self.assertEqual(result_a, expected)
        assign = func.attributes['bo']
        self.assertEqual(assign.name, 'bo')
        assign_test1 = model.Assign('bo')
        assign_test1.add_data(0, '__builtin__.bool', '    bo = True', None)
        expected_data = assign_test1.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertTrue(result_data.is_native)
        self.assertFalse(result_data.from_import)
        #Assign: 2
        assign = func.attributes['obj']
        self.assertEqual(assign.name, 'obj')
        assign_test2 = model.Assign('obj')
        assign_test2.add_data(0, model.late_resolution,
                              '    obj = os.path', _ast.Attribute)
        expected_data = assign_test2.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertFalse(result_data.is_native)
        #Assign: 3
        assign = func.attributes['di']
        self.assertEqual(assign.name, 'di')
        assign_test2 = model.Assign('di')
        assign_test2.add_data(0, model.late_resolution,
                              '    di = Test()', _ast.Call)
        expected_data = assign_test2.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertFalse(result_data.is_native)
        self.assertFalse(result_data.from_import)
        #Assign: 4
        assign = func.attributes['another']
        self.assertEqual(assign.name, 'another')
        assign_test2 = model.Assign('another')
        assign_test2.add_data(0, model.late_resolution,
                              '    another = obj', _ast.Name)
        expected_data = assign_test2.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertFalse(result_data.is_native)

    def test_simple_class_data(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        clazz = module.classes['Test']
        result_f = clazz.functions.keys()
        result_f.sort()
        result_a = clazz.attributes.keys()
        result_a.sort()
        functions = ['__init__', 'my_function', 'func_args']
        attrs = ['x', 'var']
        functions.sort()
        attrs.sort()
        self.assertEqual(result_f, functions)
        self.assertEqual(result_a, attrs)
        self.assertEqual(clazz.bases, {'object': None})

    def test_simple_class_attrs(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        clazz = module.classes['Test']
        attr_x = clazz.attributes['x']
        attr_var = clazz.attributes['var']
        type_x = attr_x.data[0]
        type_var = attr_var.data[0]

        self.assertEqual(type_x.data_type, '__builtin__.dict')
        self.assertEqual(type_x.line_content, '        self.x = {}')
        self.assertEqual(type_x.operation, None)
        self.assertEqual(type_x.from_import, False)
        self.assertEqual(type_x.is_native, True)
        self.assertEqual(type_var.data_type, '__builtin__.float')
        self.assertEqual(type_var.line_content, '        self.var = 4.5')
        self.assertEqual(type_var.operation, None)
        self.assertEqual(type_var.from_import, False)
        self.assertEqual(type_var.is_native, True)

    def test_attrs_in_class_func(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        clazz = module.classes['Test']
        func = clazz.functions['my_function']

        self.assertEqual(func.args, {})
        self.assertEqual(func.decorators, [])
        self.assertEqual(func.return_type, [])
        self.assertEqual(func.name, 'my_function')
        self.assertEqual(func.functions, {})
        #Assign
        result_a = func.attributes.keys()
        result_a.sort()
        expected = ['code', 'my_var']
        self.assertEqual(result_a, expected)
        #Assing: code
        assign = func.attributes['code']
        self.assertEqual(assign.name, 'code')
        assign_test1 = model.Assign('code')
        assign_test1.add_data(0, '__builtin__.str', "        code = 'string'",
                              None)
        expected_data = assign_test1.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertTrue(result_data.is_native)
        self.assertFalse(result_data.from_import)
        #Assing: my_var
        assign = func.attributes['my_var']
        self.assertEqual(assign.name, 'my_var')
        assign_test1 = model.Assign('my_var')
        assign_test1.add_data(0, '__builtin__.str',
                              "            my_var = 'inside if'", None)
        expected_data = assign_test1.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertTrue(result_data.is_native)
        self.assertFalse(result_data.from_import)

    def test_attrs_in_class_func_extended(self):
        module = self.analyzer.analyze(SOURCE_ANALYZER_NATIVE)

        clazz = module.classes['Test']
        func = clazz.functions['func_args']

        #Args
        args_names = ['var', 'inte', 'num', 'li', 'arggg', 'kwarggg']
        args_names.sort()
        func_args = func.args.keys()
        func_args.sort()
        self.assertEqual(func_args, args_names)
        #For: var
        type_var = model._TypeData(0, model.late_resolution, None, None)
        func_arg_obj = func.args['var']
        type_arg_func = func_arg_obj.data[0]
        self.assertEqual(func_arg_obj.name, 'var')
        self.assertEqual(type_arg_func.data_type, type_var.data_type)
        self.assertEqual(type_arg_func.line_content, type_var.line_content)
        self.assertEqual(type_arg_func.operation, type_var.operation)
        self.assertFalse(type_arg_func.is_native)
        #For: inte
        type_var = model._TypeData(0, model.late_resolution, None, None)
        func_arg_obj = func.args['inte']
        type_arg_func = func_arg_obj.data[0]
        self.assertEqual(func_arg_obj.name, 'inte')
        self.assertEqual(type_arg_func.data_type, type_var.data_type)
        self.assertEqual(type_arg_func.line_content, type_var.line_content)
        self.assertEqual(type_arg_func.operation, type_var.operation)
        self.assertFalse(type_arg_func.is_native)
        #For: num
        type_var = model._TypeData(0, '__builtin__.int', None, None)
        func_arg_obj = func.args['num']
        type_arg_func = func_arg_obj.data[0]
        self.assertEqual(func_arg_obj.name, 'num')
        self.assertEqual(type_arg_func.data_type, type_var.data_type)
        self.assertEqual(type_arg_func.line_content, type_var.line_content)
        self.assertEqual(type_arg_func.operation, type_var.operation)
        self.assertTrue(type_arg_func.is_native)
        #For: li
        type_var = model._TypeData(0, '__builtin__.str', None, None)
        func_arg_obj = func.args['li']
        type_arg_func = func_arg_obj.data[0]
        self.assertEqual(func_arg_obj.name, 'li')
        self.assertEqual(type_arg_func.data_type, type_var.data_type)
        self.assertEqual(type_arg_func.line_content, type_var.line_content)
        self.assertEqual(type_arg_func.operation, type_var.operation)
        self.assertTrue(type_arg_func.is_native)
        #For: arggg
        type_var = model._TypeData(0, '__builtin__.list', None, None)
        func_arg_obj = func.args['arggg']
        type_arg_func = func_arg_obj.data[0]
        self.assertEqual(func_arg_obj.name, 'arggg')
        self.assertEqual(type_arg_func.data_type, type_var.data_type)
        self.assertEqual(type_arg_func.line_content, type_var.line_content)
        self.assertEqual(type_arg_func.operation, type_var.operation)
        self.assertTrue(type_arg_func.is_native)
        #For: kwarggg
        type_var = model._TypeData(0, '__builtin__.dict', None, None)
        func_arg_obj = func.args['kwarggg']
        type_arg_func = func_arg_obj.data[0]
        self.assertEqual(func_arg_obj.name, 'kwarggg')
        self.assertEqual(type_arg_func.data_type, type_var.data_type)
        self.assertEqual(type_arg_func.line_content, type_var.line_content)
        self.assertEqual(type_arg_func.operation, type_var.operation)
        self.assertTrue(type_arg_func.is_native)

        #Decorators
        self.assertEqual(func.decorators, [])
        #Return Type
        self.assertEqual(func.return_type, [])
        #Attributes
        self.assertEqual(func.name, 'func_args')
        self.assertEqual(func.functions, {})
        #Assign
        result_a = func.attributes.keys()
        expected = ['nothing']
        self.assertEqual(result_a, expected)
        assign = func.attributes['nothing']
        self.assertEqual(assign.name, 'nothing')
        assign_test1 = model.Assign('nothing')
        assign_test1.add_data(0, '__builtin__.bool', "        nothing = False",
                              None)
        expected_data = assign_test1.data[0]
        result_data = assign.data[0]
        self.assertEqual(result_data.data_type, expected_data.data_type)
        self.assertEqual(result_data.line_content, expected_data.line_content)
        self.assertEqual(result_data.operation, expected_data.operation)
        self.assertTrue(result_data.is_native)
        self.assertFalse(result_data.from_import)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import
from __future__ import unicode_literals

import unittest
import os

from ninja_ide.core import file_manager


CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))


class FileManagerTestCase(unittest.TestCase):

    def setUp(self):
        global CURRENT_DIR
        self.examples_dir = os.path.join(CURRENT_DIR, 'examples')

    def test_read_file_content(self):
        filename = os.path.join(self.examples_dir, 'file_for_tests.py')
        content = file_manager.read_file_content(filename)
        expected = ("# -*- coding: utf-8 -*-\n\nprint 'testing'\n"
                    "print 'ñandú testing'\n").encode('utf-8')
        self.assertEqual(content, expected)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import os

from PyQt4.QtNetwork import QLocalSocket


file_delimiter = '<-nf>'
project_delimiter = '<-np>'


def is_running():
    local_socket = QLocalSocket()
    local_socket.connectToServer("ninja_ide")
    if local_socket.state():
        result = (True, local_socket)
    else:
        result = (False, local_socket)
    return result


def send_data(socket, filenames, projects_path, linenos):
    global file_delimiter
    global project_delimiter
    file_with_nro = ['%s:%i' % (os.path.abspath(f[0]), f[1] - 1)
                     for f in zip(filenames, linenos)]
    file_without_nro = ['%s:%i' % (os.path.abspath(f), 0)
                        for f in filenames[len(linenos):]]
    filenames = file_with_nro + file_without_nro
    files = file_delimiter.join(filenames)
    projects = project_delimiter.join(projects_path)
    data = files + project_delimiter + projects
    data_sended = False
    try:
        result = socket.write(data)
        socket.flush()
        socket.close()
        if result >= 0:
            data_sended = True
    except:
        data_sended = False
    return data_sended



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import

import math
import re

from PyQt4.QtGui import QWidget
from PyQt4.QtGui import QBrush
from PyQt4.QtGui import QLinearGradient
from PyQt4.QtGui import QPixmap
from PyQt4.QtGui import QColor
from PyQt4.QtGui import QPolygonF
from PyQt4.QtGui import QFontMetrics
from PyQt4.QtGui import QPainter
from PyQt4.QtCore import Qt
from PyQt4.QtCore import QPointF

from ninja_ide import resources
from ninja_ide.core import settings
from ninja_ide.gui.editor import helpers


#based on: http://john.nachtimwald.com/2009/08/15/qtextedit-with-line-numbers/
#(MIT license)


class SidebarWidget(QWidget):

    def __init__(self, editor):
        QWidget.__init__(self, editor)
        self.edit = editor
        self.highest_line = 0
        self.foldArea = 15
        self.rightArrowIcon = QPixmap()
        self.downArrowIcon = QPixmap()
        self.pat = re.compile(
            "(\s)*def |(\s)*class |(\s)*if |(\s)*while |"
            "(\s)*else:|(\s)*elif |(\s)*for |"
            "(\s)*try:|(\s)*except:|(\s)*except |(\s)*#begin-fold:")
        self.patNotPython = re.compile('(\s)*#begin-fold:|(.)*{')
        self._foldedBlocks = []
        self._breakpoints = []
        self._bookmarks = []
        self._pep8Lines = []
        self._errorsLines = []
        self._migrationLines = []

    def update_area(self):
        maxLine = math.ceil(math.log10(self.edit.blockCount()))
        width = QFontMetrics(
            self.edit.document().defaultFont()).width('0' * int(maxLine)) \
                + 10 + self.foldArea
        if self.width() != width:
            self.setFixedWidth(width)
            self.edit.setViewportMargins(width, 0, 0, 0)
        self.update()

    def update(self, *args):
        QWidget.update(self, *args)

    def pep8_check_lines(self, lines):
        self._pep8Lines = lines

    def static_errors_lines(self, lines):
        self._errorsLines = lines

    def migration_lines(self, lines):
        self._migrationLines = lines

    def code_folding_event(self, lineNumber):
        if self._is_folded(lineNumber):
            self._fold(lineNumber)
        else:
            self._unfold(lineNumber)

        self.edit.update()
        self.update()

    def _fold(self, lineNumber):
        startBlock = self.edit.document().findBlockByNumber(lineNumber - 1)
        endPos = self._find_fold_closing(startBlock)
        endBlock = self.edit.document().findBlockByNumber(endPos)

        block = startBlock.next()
        while block.isValid() and block != endBlock:
            block.setVisible(False)
            block.setLineCount(0)
            block = block.next()

        self._foldedBlocks.append(startBlock.blockNumber())
        self.edit.document().markContentsDirty(startBlock.position(), endPos)

    def _unfold(self, lineNumber):
        startBlock = self.edit.document().findBlockByNumber(lineNumber - 1)
        endPos = self._find_fold_closing(startBlock)
        endBlock = self.edit.document().findBlockByNumber(endPos)

        block = startBlock.next()
        while block.isValid() and block != endBlock:
            block.setVisible(True)
            block.setLineCount(block.layout().lineCount())
            endPos = block.position() + block.length()
            if block.blockNumber() in self._foldedBlocks:
                close = self._find_fold_closing(block)
                block = self.edit.document().findBlockByNumber(close)
            else:
                block = block.next()

        self._foldedBlocks.remove(startBlock.blockNumber())
        self.edit.document().markContentsDirty(startBlock.position(), endPos)

    def _is_folded(self, line):
        block = self.edit.document().findBlockByNumber(line)
        if not block.isValid():
            return False
        return block.isVisible()

    def _find_fold_closing(self, block):
        text = block.text()
        pat = re.compile('(\s)*#begin-fold:')
        patBrace = re.compile('(.)*{$')
        if pat.match(text):
            return self._find_fold_closing_label(block)
        elif patBrace.match(text):
            return self._find_fold_closing_brace(block)

        spaces = helpers.get_leading_spaces(text)
        pat = re.compile('^\s*$|^\s*#')
        block = block.next()
        while block.isValid():
            text2 = block.text()
            if not pat.match(text2):
                spacesEnd = helpers.get_leading_spaces(text2)
                if len(spacesEnd) <= len(spaces):
                    if pat.match(block.previous().text()):
                        return block.previous().blockNumber()
                    else:
                        return block.blockNumber()
            block = block.next()
        return block.previous().blockNumber()

    def _find_fold_closing_label(self, block):
        text = block.text()
        label = text.split(':')[1]
        block = block.next()
        pat = re.compile('\s*#end-fold:' + label)
        while block.isValid():
            if pat.match(block.text()):
                return block.blockNumber() + 1
            block = block.next()
        return block.blockNumber()

    def _find_fold_closing_brace(self, block):
        block = block.next()
        openBrace = 1
        while block.isValid():
            openBrace += block.text().count('{')
            openBrace -= block.text().count('}')
            if openBrace == 0:
                return block.blockNumber() + 1
            elif openBrace < 0:
                return block.blockNumber()
            block = block.next()
        return block.blockNumber()

    def paintEvent(self, event):
        page_bottom = self.edit.viewport().height()
        font_metrics = QFontMetrics(self.edit.document().defaultFont())
        current_block = self.edit.document().findBlock(
            self.edit.textCursor().position())
        pattern = self.pat if self.edit.lang == "python" else self.patNotPython

        painter = QPainter(self)
        background = resources.CUSTOM_SCHEME.get('sidebar-background',
            resources.COLOR_SCHEME['sidebar-background'])
        foreground = resources.CUSTOM_SCHEME.get('sidebar-foreground',
            resources.COLOR_SCHEME['sidebar-foreground'])
        pep8color = resources.CUSTOM_SCHEME.get('pep8-underline',
            resources.COLOR_SCHEME['pep8-underline'])
        errorcolor = resources.CUSTOM_SCHEME.get('error-underline',
            resources.COLOR_SCHEME['error-underline'])
        migrationcolor = resources.CUSTOM_SCHEME.get('migration-underline',
            resources.COLOR_SCHEME['migration-underline'])
        painter.fillRect(self.rect(), QColor(background))

        block = self.edit.firstVisibleBlock()
        viewport_offset = self.edit.contentOffset()
        line_count = block.blockNumber()
        painter.setFont(self.edit.document().defaultFont())
        while block.isValid():
            line_count += 1
            # The top left position of the block in the document
            position = self.edit.blockBoundingGeometry(block).topLeft() + \
                viewport_offset
            # Check if the position of the block is outside of the visible area
            if position.y() > page_bottom:
                break

            # Set the Painter Pen depending on special lines
            error = False
            if settings.CHECK_STYLE and \
               ((line_count - 1) in self._pep8Lines):
                painter.setPen(QColor(pep8color))
                font = painter.font()
                font.setItalic(True)
                font.setUnderline(True)
                painter.setFont(font)
                error = True
            elif settings.FIND_ERRORS and \
                 ((line_count - 1) in self._errorsLines):
                painter.setPen(QColor(errorcolor))
                font = painter.font()
                font.setItalic(True)
                font.setUnderline(True)
                painter.setFont(font)
                error = True
            elif settings.SHOW_MIGRATION_TIPS and \
                 ((line_count - 1) in self._migrationLines):
                painter.setPen(QColor(migrationcolor))
                font = painter.font()
                font.setItalic(True)
                font.setUnderline(True)
                painter.setFont(font)
                error = True
            else:
                painter.setPen(QColor(foreground))

            # We want the line number for the selected line to be bold.
            bold = False
            if block == current_block:
                bold = True
                font = painter.font()
                font.setBold(True)
                painter.setFont(font)

            # Draw the line number right justified at the y position of the
            # line. 3 is a magic padding number. drawText(x, y, text).
            if block.isVisible():
                painter.drawText(self.width() - self.foldArea -
                    font_metrics.width(str(line_count)) - 3,
                    round(position.y()) + font_metrics.ascent() +
                    font_metrics.descent() - 1,
                    str(line_count))

            # Remove the bold style if it was set previously.
            if bold:
                font = painter.font()
                font.setBold(False)
                painter.setFont(font)
            if error:
                font = painter.font()
                font.setItalic(False)
                font.setUnderline(False)
                painter.setFont(font)

            block = block.next()

        self.highest_line = line_count

        #Code Folding
        xofs = self.width() - self.foldArea
        painter.fillRect(xofs, 0, self.foldArea, self.height(),
                QColor(resources.CUSTOM_SCHEME.get('fold-area',
                resources.COLOR_SCHEME['fold-area'])))
        if self.foldArea != self.rightArrowIcon.width():
            polygon = QPolygonF()

            self.rightArrowIcon = QPixmap(self.foldArea, self.foldArea)
            self.rightArrowIcon.fill(Qt.transparent)
            self.downArrowIcon = QPixmap(self.foldArea, self.foldArea)
            self.downArrowIcon.fill(Qt.transparent)

            polygon.append(QPointF(self.foldArea * 0.4, self.foldArea * 0.25))
            polygon.append(QPointF(self.foldArea * 0.4, self.foldArea * 0.75))
            polygon.append(QPointF(self.foldArea * 0.8, self.foldArea * 0.5))
            iconPainter = QPainter(self.rightArrowIcon)
            iconPainter.setRenderHint(QPainter.Antialiasing)
            iconPainter.setPen(Qt.NoPen)
            iconPainter.setBrush(QColor(
                resources.CUSTOM_SCHEME.get('fold-arrow',
                resources.COLOR_SCHEME['fold-arrow'])))
            iconPainter.drawPolygon(polygon)

            polygon.clear()
            polygon.append(QPointF(self.foldArea * 0.25, self.foldArea * 0.4))
            polygon.append(QPointF(self.foldArea * 0.75, self.foldArea * 0.4))
            polygon.append(QPointF(self.foldArea * 0.5, self.foldArea * 0.8))
            iconPainter = QPainter(self.downArrowIcon)
            iconPainter.setRenderHint(QPainter.Antialiasing)
            iconPainter.setPen(Qt.NoPen)
            iconPainter.setBrush(QColor(
                resources.CUSTOM_SCHEME.get('fold-arrow',
                resources.COLOR_SCHEME['fold-arrow'])))
            iconPainter.drawPolygon(polygon)

        block = self.edit.firstVisibleBlock()
        while block.isValid():
            position = self.edit.blockBoundingGeometry(
                block).topLeft() + viewport_offset
            #Check if the position of the block is outside of the visible area
            if position.y() > page_bottom:
                break

            if pattern.match(block.text()) and block.isVisible():
                if block.blockNumber() in self._foldedBlocks:
                    painter.drawPixmap(xofs, round(position.y()),
                        self.rightArrowIcon)
                else:
                    painter.drawPixmap(xofs, round(position.y()),
                        self.downArrowIcon)
            #Add Bookmarks and Breakpoint
            elif block.blockNumber() in self._breakpoints:
                linear_gradient = QLinearGradient(
                    xofs, round(position.y()),
                    xofs + self.foldArea, round(position.y()) + self.foldArea)
                linear_gradient.setColorAt(0, QColor(255, 11, 11))
                linear_gradient.setColorAt(1, QColor(147, 9, 9))
                painter.setRenderHints(QPainter.Antialiasing, True)
                painter.setPen(Qt.NoPen)
                painter.setBrush(QBrush(linear_gradient))
                painter.drawEllipse(
                    xofs + 1,
                    round(position.y()) + 6,
                    self.foldArea - 1, self.foldArea - 1)
            elif block.blockNumber() in self._bookmarks:
                linear_gradient = QLinearGradient(
                    xofs, round(position.y()),
                    xofs + self.foldArea, round(position.y()) + self.foldArea)
                linear_gradient.setColorAt(0, QColor(13, 62, 243))
                linear_gradient.setColorAt(1, QColor(5, 27, 106))
                painter.setRenderHints(QPainter.Antialiasing, True)
                painter.setPen(Qt.NoPen)
                painter.setBrush(QBrush(linear_gradient))
                painter.drawRoundedRect(
                    xofs + 1,
                    round(position.y()) + 6,
                    self.foldArea - 2, self.foldArea - 1,
                    3, 3)

            block = block.next()

        painter.end()
        QWidget.paintEvent(self, event)

    def mousePressEvent(self, event):
        if self.foldArea > 0:
            xofs = self.width() - self.foldArea
            font_metrics = QFontMetrics(self.edit.document().defaultFont())
            fh = font_metrics.lineSpacing()
            ys = event.posF().y()
            lineNumber = 0

            if event.pos().x() > xofs:
                pattern = self.pat
                if self.edit.lang != "python":
                    pattern = self.patNotPython
                block = self.edit.firstVisibleBlock()
                viewport_offset = self.edit.contentOffset()
                page_bottom = self.edit.viewport().height()
                while block.isValid():
                    position = self.edit.blockBoundingGeometry(
                        block).topLeft() + viewport_offset
                    if position.y() > page_bottom:
                        break
                    if position.y() < ys and (position.y() + fh) > ys and \
                      pattern.match(str(block.text())):
                        lineNumber = block.blockNumber() + 1
                        break
                    elif position.y() < ys and (position.y() + fh) > ys and \
                      event.button() == Qt.LeftButton:
                        line = block.blockNumber()
                        if line in self._breakpoints:
                            self._breakpoints.remove(line)
                        else:
                            self._breakpoints.append(line)
                        self.update()
                        break
                    elif position.y() < ys and (position.y() + fh) > ys and \
                      event.button() == Qt.RightButton:
                        line = block.blockNumber()
                        if line in self._bookmarks:
                            self._bookmarks.remove(line)
                        else:
                            self._bookmarks.append(line)
                        self.update()
                        break
                    block = block.next()
                self._save_breakpoints_bookmarks()
            if lineNumber > 0:
                self.code_folding_event(lineNumber)

    def _save_breakpoints_bookmarks(self):
        if self._bookmarks and self.edit.ID != "":
            settings.BOOKMARKS[self.edit.ID] = self._bookmarks
        elif self.edit.ID in settings.BOOKMARKS:
            settings.BOOKMARKS.pop(self.edit.ID)
        if self._breakpoints and self.edit.ID != "":
            settings.BREAKPOINTS[self.edit.ID] = self._breakpoints
        elif self.edit.ID in settings.BREAKPOINTS:
            settings.BREAKPOINTS.pop(self.edit.ID)

    def set_breakpoint(self, lineno):
        if lineno in self._breakpoints:
            self._breakpoints.remove(lineno)
        else:
            self._breakpoints.append(lineno)
        self.update()
        self._save_breakpoints_bookmarks()

    def set_bookmark(self, lineno):
        if lineno in self._bookmarks:
            self._bookmarks.remove(lineno)
        else:
            self._bookmarks.append(lineno)
        self.update()
        self._save_breakpoints_bookmarks()



# -*- coding: utf-8 -*-
#
# This file is part of NINJA-IDE (http://ninja-ide.org).
#
# NINJA-IDE is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
#
# NINJA-IDE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with NINJA-IDE; If not, see <http://www.gnu.org/licenses/>.
from __future__ import absolute_import

from PyQt4.QtGui import QIcon
from PyQt4.QtGui import QKeySequence
from PyQt4.QtGui import QWheelEvent
from PyQt4.QtCore import SIGNAL
from PyQt4.QtCore import QObject
from PyQt4.QtCore import QPoint
from PyQt4.QtCore import Qt

from ninja_ide import resources


class MenuView(QObject):

    def __init__(self, menuView, toolbar, ide):
        QObject.__init__(self)
        self.__ide = ide

        self.hideConsoleAction = menuView.addAction(
            (self.trUtf8("Show/Hide &Console (%s)") %
                resources.get_shortcut("Hide-misc").toString(
                    QKeySequence.NativeText)))
        self.hideConsoleAction.setCheckable(True)
        self.hideEditorAction = menuView.addAction(
            (self.trUtf8("Show/Hide &Editor (%s)") %
                resources.get_shortcut("Hide-editor").toString(
                    QKeySequence.NativeText)))
        self.hideEditorAction.setCheckable(True)
        self.hideAllAction = menuView.addAction(
            (self.trUtf8("Show/Hide &All (%s)") %
                resources.get_shortcut("Hide-all").toString(
                    QKeySequence.NativeText)))
        self.hideAllAction.setCheckable(True)
        self.hideExplorerAction = menuView.addAction(
            (self.trUtf8("Show/Hide &Explorer (%s)") %
                resources.get_shortcut("Hide-explorer").toString(
                    QKeySequence.NativeText)))
        self.hideExplorerAction.setCheckable(True)
        self.hideToolbarAction = menuView.addAction(
            self.trUtf8("Show/Hide &Toolbar"))
        self.hideToolbarAction.setCheckable(True)
        self.fullscreenAction = menuView.addAction(
            QIcon(resources.IMAGES['fullscreen']),
            (self.trUtf8("Full Screen &Mode (%s)") %
                resources.get_shortcut("Full-screen").toString(
                    QKeySequence.NativeText)))
        menuView.addSeparator()
        splitTabHAction = menuView.addAction(
            QIcon(resources.IMAGES['splitH']),
            (self.trUtf8("Split Tabs Horizontally (%s)") %
                resources.get_shortcut("Split-horizontal").toString(
                    QKeySequence.NativeText)))
        splitTabVAction = menuView.addAction(
            QIcon(resources.IMAGES['splitV']),
            (self.trUtf8("Split Tabs Vertically (%s)") %
                resources.get_shortcut("Split-vertical").toString(
                    QKeySequence.NativeText)))
        followModeAction = menuView.addAction(
            QIcon(resources.IMAGES['follow']),
            (self.trUtf8("Follow Mode (%s)") %
                resources.get_shortcut("Follow-mode").toString(
                    QKeySequence.NativeText)))
        groupTabsAction = menuView.addAction(
            self.trUtf8("Group Tabs by Project"))
        deactivateGroupTabsAction = menuView.addAction(
            self.trUtf8("Deactivate Group Tabs"))
        menuView.addSeparator()
        #Zoom
        zoomInAction = menuView.addAction(QIcon(resources.IMAGES['zoom-in']),
            self.trUtf8("Zoom &In (Ctrl+Wheel-Up)"))
        zoomOutAction = menuView.addAction(QIcon(resources.IMAGES['zoom-out']),
            self.trUtf8("Zoom &Out (Ctrl+Wheel-Down)"))
        menuView.addSeparator()
        fadeInAction = menuView.addAction(
            self.trUtf8("Fade In (Alt+Wheel-Up)"))
        fadeOutAction = menuView.addAction(
            self.trUtf8("Fade Out (Alt+Wheel-Down)"))

        self.toolbar_items = {
            'splitv': splitTabVAction,
            'splith': splitTabHAction,
            'follow-mode': followModeAction,
            'zoom-in': zoomInAction,
            'zoom-out': zoomOutAction}

        self.connect(self.hideConsoleAction, SIGNAL("triggered()"),
            self.__ide.actions.view_misc_visibility)
        self.connect(self.hideEditorAction, SIGNAL("triggered()"),
            self.__ide.actions.view_main_visibility)
        self.connect(self.hideExplorerAction, SIGNAL("triggered()"),
            self.__ide.actions.view_explorer_visibility)
        self.connect(self.hideAllAction, SIGNAL("triggered()"),
            self.__ide.actions.hide_all)
        self.connect(self.fullscreenAction, SIGNAL("triggered()"),
            self.__ide.actions.fullscreen_mode)

        self.connect(splitTabHAction, SIGNAL("triggered()"),
            lambda: self.__ide.mainContainer.split_tab(True))
        self.connect(splitTabVAction, SIGNAL("triggered()"),
            lambda: self.__ide.mainContainer.split_tab(False))
        QObject.connect(followModeAction, SIGNAL("triggered()"),
            self.__ide.mainContainer.show_follow_mode)
        self.connect(zoomInAction, SIGNAL("triggered()"),
            self.zoom_in_editor)
        self.connect(zoomOutAction, SIGNAL("triggered()"),
            self.zoom_out_editor)
        self.connect(fadeInAction, SIGNAL("triggered()"), self._fade_in)
        self.connect(fadeOutAction, SIGNAL("triggered()"), self._fade_out)
        self.connect(self.hideToolbarAction, SIGNAL("triggered()"),
            self._hide_show_toolbar)
        self.connect(groupTabsAction, SIGNAL("triggered()"),
            self.__ide.actions.group_tabs_together)
        self.connect(deactivateGroupTabsAction, SIGNAL("triggered()"),
            self.__ide.actions.deactivate_tabs_groups)

        #Set proper MenuView checks values:
        self.hideAllAction.setChecked(True)
        self.hideConsoleAction.setChecked(False)
        self.hideEditorAction.setChecked(True)
        self.hideExplorerAction.setChecked(True)
        self.hideToolbarAction.setChecked(True)

    def _hide_show_toolbar(self):
        if self.__ide.toolbar.isVisible():
            self.__ide.toolbar.hide()
        else:
            self.__ide.toolbar.show()

    def zoom_in_editor(self):
        editor = self.__ide.mainContainer.get_actual_editor()
        if editor:
            editor.zoom_in()

    def zoom_out_editor(self):
        editor = self.__ide.mainContainer.get_actual_editor()
        if editor:
            editor.zoom_out()

    def _fade_in(self):
        event = QWheelEvent(QPoint(), 120, Qt.NoButton, Qt.AltModifier)
        self.__ide.wheelEvent(event)

    def _fade_out(self):
        event = QWheelEvent(QPoint(), -120, Qt.NoButton, Qt.AltModifier)
        self.__ide.wheelEvent(event)
